/* PSPFTPD */

#include "std.h"
#include "util.h"
#include "ftpd.h"
#include "psp_init.h"

#include "nlh.h" // net lib helper

//NOTE: kernel mode module flag and kernel mode thread are both required
PSP_MODULE_INFO("PSP-FTPD", 0x1000, 1, 1); /* 0x1000 REQUIRED!!! */
PSP_MAIN_THREAD_ATTR(0); /* 0 REQUIRED!!! */
PSP_MAIN_THREAD_STACK_SIZE_KB(32); /* smaller stack for kernel thread */

static void 
DoInetNetFtpd(void)
{
    u32 err;
    int state;
    char szMyIPAddr[32];
    int connectionConfig = -1;
    PICKER pickConn; // connection picker
    int iNetIndex;
    int iPick;


    // nlhInit must be called from user thread for DHCP to work
    err = nlhInit();
    if (err != 0) {
      pspDebugScreenPrintf("nlhInit: %d !\n", err);
      nlhTerm();
      return;
    }

    // enumerate connections
    my_initpicker(&pickConn, "> Choose a connection:             X OK    [] CANCEL");

    for (iNetIndex = 1; iNetIndex < 100; iNetIndex++) // skip the 0th connection
    {
      char data[128];
      char name[128];
      char detail[128];
      if (sceUtilityCheckNetParam(iNetIndex) != 0) break;  // no more
      sceUtilityGetNetParam(iNetIndex, 0, name);

      sceUtilityGetNetParam(iNetIndex, 1, data);
      strcpy(detail, "SSID=");
      strcat(detail, data);

      sceUtilityGetNetParam(iNetIndex, 4, data);
      if (data[0]) {
        // not DHCP
        sceUtilityGetNetParam(iNetIndex, 5, data);
        strcat(detail, " IPADDR=");
        strcat(detail, data);
      } else {
        strcat(detail, " DHCP");
      }

my_addpick(&pickConn, name, detail, (u32)iNetIndex);
if (pickConn.pick_count >= MAX_PICK) break;  // no more
}

if (pickConn.pick_count == 0) {
pspDebugScreenPrintf("No wifi connections, please create one !");
goto close_net;
}

pickConn.pick_start = 0;
iPick = my_picker(&pickConn);
if (iPick == -1) goto close_net;
connectionConfig = (int)(pickConn.picks[iPick].userData);


err = sceNetApctlConnect(connectionConfig);
if (err != 0) {
goto close_net;
}

state = 0;
err = sceNetApctlGetState(&state);
if (err != 0) {
goto close_connection;
}

pspDebugScreenClear();
pspDebugScreenSetXY(0,0);
pspDebugScreenPrintf("Trying to connect to %s\n", pickConn.picks[iPick].szBig);

// 4 connected with IP address
while (1) {
char szT[32];
int state;

err = sceNetApctlGetState(&state);
if (err != 0)
{
pspDebugScreenPrintf("Connection failed !\n");
goto close_connection;
}
if (state >= 0 && state <= 4) {
  // cute little status bar
  memset(szT, '.', 10);
  szT[(state+1)*2] = '\0';
} else {
  sprintf(szT, "state=%d", state);
}
sceKernelDelayThread(1000000); 
pspDebugScreenSetXY(0,1);
pspDebugScreenPrintf("%s", szT);

// 0 - idle
// 1,2 - starting up
// 3 - waiting for dynamic IP
// 4 - got IP - usable
if (state == 4) break;  // connected with static IP
}

// get IP address
if (sceNetApctlGetInfo(8, szMyIPAddr) != 0) {
strcpy(szMyIPAddr, "unknown IP address");
}

ftpdLoop(szMyIPAddr);

close_connection:
  err = sceNetApctlDisconnect();

close_net:
  pspDebugScreenClear();
  pspDebugScreenPrintf("Closing connection ...");

  nlhTerm();
  sceKernelDelayThread(1000000); 
}

int 
main(int argc, char **argv)
{
  pspDebugScreenInit();

  psp_init_stuff();

  pspDebugScreenPrintf("PSP-FTPD v0.3 by Zx-81\n\n");
  pspDebugScreenPrintf("Based on the work of Raf, Krazy, Pet and Theno\n\n");
  pspDebugScreenPrintf("Please wait now loading !\n");
  sceKernelDelayThread(2000000); 

  if (nlhLoadDrivers(&module_info) != 0) {
    pspDebugScreenPrintf("unable to load wifi drivers !");
    return 0;
  }

  DoInetNetFtpd();

  psp_exit(0);

  return 0;
}

